import { Random, Seed } from "@fuzz-utils/random";
import { Shrink } from "@fuzz-utils/shrink";
declare type Filter = {
    __FILTER__: true;
};
declare type Generator<T, U> = (size: number, seed: Seed) => [Random<T>, Shrink<T>, Seed, FilterMap<T, U>];
export declare class RoseTree<T, U> {
    private pair;
    private shrink;
    private filterMap;
    constructor(pair: [T, U], shrink: Shrink<T, T>, filterMap: FilterMap<T, U>);
    value(): U;
    children(): IterableIterator<RoseTree<T, U>>;
}
declare class FilterMap<T, U> {
    apply: (t: T) => U | Filter;
    static tuple<A, B>(arr: [FilterMap<A, B>]): FilterMap<[A], [B]>;
    static tuple<A, B, C, D>(arr: [FilterMap<A, B>, FilterMap<C, D>]): FilterMap<[A, C], [B, D]>;
    static tuple<A, B, C, D, E, F>(arr: [FilterMap<A, B>, FilterMap<C, D>, FilterMap<E, F>]): FilterMap<[A, C, E], [B, D, F]>;
    static tuple<A, B, C, D, E, F, G, H>(arr: [FilterMap<A, B>, FilterMap<C, D>, FilterMap<E, F>, FilterMap<G, H>]): FilterMap<[A, C, E, G], [B, D, F, H]>;
    static tuple<A, B, C, D, E, F, G, H, I, J>(arr: [FilterMap<A, B>, FilterMap<C, D>, FilterMap<E, F>, FilterMap<G, H>, FilterMap<I, J>]): FilterMap<[A, C, E, G, I], [B, D, F, H, J]>;
    static tuple<A, B, C, D, E, F, G, H, I, J, K, L>(arr: [FilterMap<A, B>, FilterMap<C, D>, FilterMap<E, F>, FilterMap<G, H>, FilterMap<I, J>, FilterMap<K, L>]): FilterMap<[A, C, E, G, I, K], [B, D, F, H, J, L]>;
    static tuple<T, U>(arr: FilterMap<T, U>[]): FilterMap<T[], U[]>;
    static object<T>(obj: {
        [K in keyof T]: FilterMap<any, T[K]>;
    }): FilterMap<any, T>;
    constructor(apply: (t: T) => U | Filter);
    filter(fn: (u: U) => boolean): FilterMap<T, U>;
    map<V>(fn: (u: U) => V): FilterMap<T, V>;
    toArray(): FilterMap<T[], U[]>;
    maybe(): FilterMap<T | undefined, U | undefined>;
    nullable(): FilterMap<T | null, U | null>;
}
export declare class Fuzz<T, U> {
    readonly generator: Generator<T, U>;
    static is(obj: any): obj is Fuzz<any, any>;
    static from<T>(random: Random<T>, shrink: Shrink<T>): Fuzz<T, T>;
    constructor(generator: Generator<T, U>);
    /**
     * Maps the inner value to a new value.
     * @param fn Map function
     */
    map<V>(fn: (u: U) => V): Fuzz<T, V>;
    /**
     * Merges two fuzzers into a single fuzzer
     * @param fuzz Fuzz to be merged
     * @param fn Mapping function to merge both values
     */
    merge<V, W, X>(fuzz: Fuzz<V, W>, fn: (u: U, w: W) => X): Fuzz<[T, V], X>;
    /**
     * Filters out any value or shrinken value that does not return true.
     * @param fn Filter function
     */
    suchThat(fn: (u: U) => boolean): Fuzz<T, U>;
    filter: (fn: (u: U) => boolean) => Fuzz<T, U>;
    /**
     * The result value will not shrink.
     */
    noShrink(): Fuzz<T, U>;
    /**
     * Prevents empty values from being generated. Does not recursively apply
     * on higher order fuzzers (such as object). Empty values include anything
     * that matches `!!value === false` or `value.length === 0`.
     */
    noEmpty(): Fuzz<T, U>;
    /**
     * Changes a fuzzer from always return a value to returning a value or `undefined`.
     * 1 out of every n times, it will return `undefined`.
     * @param n the number of generated values where one value is _likely_ to be `undefined`.
     */
    maybe(n?: number): Fuzz<T | undefined, U | undefined>;
    /**
     * Changes a fuzzer from always return a value to returning a value or `null`.
     * 1 out of every n times, it will return `null`.
     * @param n the number of generated values where one value is _likely_ to be `null`.
     */
    nullable(n?: number): Fuzz<T | null, U | null>;
    /**
     * Maps a value generated by the fuzzer to another fuzzer.
     * @param fn maps
     */
    bind<V, W>(fn: (u: U) => Fuzz<V, W>): Fuzz<V, W>;
    /**
     * Forces the maximum size of the fuzzer instead of allowing it grow from zero.
     * @param maxSize Forced max size
     */
    resize(maxSize: number): Fuzz<T, U>;
    /**
     * Scales the size of the fuzzer.
     * @param fn Scaling function
     */
    scale(fn: (size: number) => number): Fuzz<T, U>;
    /**
     * Internal function. Turns the fuzzer into a `Random<RoseTree>`.
     */
    toRandomRoseTree(): Random<RoseTree<T, U>>;
}
declare class Api {
    /**
     * Wraps a plain value in a fuzzer.
     * @param value Any value.
     */
    return<T>(value: T): Fuzz<T, T>;
    from<T>(value: T | Fuzz<any, T>): Fuzz<any, T>;
    /**
     * Creates an integer fuzzer.
     */
    integer(): Fuzz<number, number>;
    /**
     * Creates positive integer fuzzer.
     */
    posInteger(): Fuzz<number, number>;
    /**
     * Creates negative integer fuzzer.
     */
    negInteger(): Fuzz<number, number>;
    /**
     * Creates a fuzzer that returns an integer with some bounds. Does not bias
     * the distribution of values.
     * @param minSize The minimum value
     * @param maxSize The maximum value
     */
    integerWithin(minSize: number, maxSize: number): Fuzz<number, number>;
    /**
     * Creates a float fuzzer.
     */
    float(): Fuzz<number, number>;
    /**
     * Creates a positive float fuzzer.
     */
    posFloat(): Fuzz<number, number>;
    /**
     * Creates a negative float fuzzer.
     */
    negFloat(): Fuzz<number, number>;
    /**
     * Creates a fuzzer that returns a float with some bounds. Does not bias
     * the distribution of values.
     * @param minSize The minimum value
     * @param maxSize The maximum value
     */
    floatWithin(minSize: number, maxSize: number): Fuzz<number, number>;
    /**
     * Creates a number fuzzer.
     */
    number(): Fuzz<number, number>;
    /**
     * Creates a positive number fuzzer.
     */
    posNumber(): Fuzz<number, number>;
    /**
     * Creates a negative number fuzzer.
     */
    negNumber(): Fuzz<number, number>;
    /**
     * Creates a fuzzer that returns a number with some bounds. Does not bias
     * the distribution of values.
     * @param minSize The minimum value
     * @param maxSize The maximum value
     */
    numberWithin(minSize: number, maxSize: number): Fuzz<number, number>;
    /**
     * Creates a boolean fuzzer.
     */
    boolean(): Fuzz<boolean, boolean>;
    /**
     * Creates a fuzzer that always returns true.
     */
    true(): Fuzz<boolean, boolean>;
    /**
     * Creates a fuzzer that always returns false.
     */
    false(): Fuzz<boolean, boolean>;
    /**
     * Creates a string fuzzer of ASCII characters.
     */
    string(): Fuzz<string, string>;
    /**
     * Creates a string fuzzer of just one ASCII character.
     */
    character(): Fuzz<string, string>;
    /**
     * Creates a UUID fuzzer. Does not shrink the value.
     */
    uuid(): Fuzz<string, string>;
    /**
     * Lazily returns a fuzzer. Useful for recursive data structures.
     * @param fn Returns a fuzzer
     */
    lazy<T>(fn: () => Fuzz<any, T>): Fuzz<any, T>;
    /**
     * Always returns null.
     */
    null(): Fuzz<null, null>;
    /**
     * Always returns undefined.
     */
    undefined(): Fuzz<undefined, undefined>;
    /**
     * Garbage.
     */
    any(): Fuzz<any, any>;
    /**
     * A higher-order fuzzer that creates an array of another fuzzer.
     * @param fuzz The members of a fuzzer.
     */
    array<T, U>(fuzz: Fuzz<T, U>): Fuzz<T[], U[]>;
    /**
     * A higher-order fuzzer that returns a fuzzer based on frequency.
     * @param contexts An array of tuples where each tuple is a frequency of being selected and a fuzzer
     */
    frequency<T, U>(contexts: [number, Fuzz<T, U>][]): Fuzz<T, U>;
    /**
     * A higher-order fuzzer that randomly selects one in given sample.
     * @param arr An array of fuzzers to be sampled from
     */
    oneOf<T, U>(arr: Fuzz<T, U>[]): Fuzz<T, U>;
    /**
     * Does an Object spread over an array of fuzzer values. Same effect as
     * something like, `{ ...objA, ...objB }`
     * @param arr An array of fuzzers to spread over
     */
    spread(arr: []): Fuzz<{}, {}>;
    spread<A, B>(arr: [Fuzz<A, B>]): Fuzz<[A], B>;
    spread<A, B, C, D>(arr: [Fuzz<A, B>, Fuzz<C, D>]): Fuzz<[A, C], B & D>;
    spread<A, B, C, D, E, F>(arr: [Fuzz<A, B>, Fuzz<C, D>, Fuzz<E, F>]): Fuzz<[A, C, E], B & D & F>;
    spread<A, B, C, D, E, F, G, H>(arr: [Fuzz<A, B>, Fuzz<C, D>, Fuzz<E, F>, Fuzz<G, H>]): Fuzz<[A, C, E, G], B & D & F & H>;
    spread<A, B, C, D, E, F, G, H, I, J>(arr: [Fuzz<A, B>, Fuzz<C, D>, Fuzz<E, F>, Fuzz<G, H>, Fuzz<I, J>]): Fuzz<[A, C, E, G, I], B & D & F & H & J>;
    spread<A, B, C, D, E, F, G, H, I, J, K, L>(arr: [Fuzz<A, B>, Fuzz<C, D>, Fuzz<E, F>, Fuzz<G, H>, Fuzz<I, J>, Fuzz<K, L>]): Fuzz<[A, C, E, G, I, K], B & D & F & H & J & L>;
    spread<T>(arr: Fuzz<any, T>[]): Fuzz<any, T>;
    /**
     * Similar to array, but is a finite length of pre-defined fuzzers.
     * Shrinks the individual values, but never the length.
     * @param arr A list of fuzzers.
     */
    tuple<A, B>(arr: [Fuzz<A, B>]): Fuzz<[A], [B]>;
    tuple<A, B, C, D>(arr: [Fuzz<A, B>, Fuzz<C, D>]): Fuzz<[A, C], [B, D]>;
    tuple<A, B, C, D, E, F>(arr: [Fuzz<A, B>, Fuzz<C, D>, Fuzz<E, F>]): Fuzz<[A, C, E], [B, D, F]>;
    tuple<A, B, C, D, E, F, G, H>(arr: [Fuzz<A, B>, Fuzz<C, D>, Fuzz<E, F>, Fuzz<G, H>]): Fuzz<[A, C, E, G], [B, D, F, H]>;
    tuple<A, B, C, D, E, F, G, H, I, J>(arr: [Fuzz<A, B>, Fuzz<C, D>, Fuzz<E, F>, Fuzz<G, H>, Fuzz<I, J>]): Fuzz<[A, C, E, G, I], [B, D, F, H, J]>;
    tuple<A, B, C, D, E, F, G, H, I, J, K, L>(arr: [Fuzz<A, B>, Fuzz<C, D>, Fuzz<E, F>, Fuzz<G, H>, Fuzz<I, J>, Fuzz<K, L>]): Fuzz<[A, C, E, G, I, K], [B, D, F, H, J, L]>;
    tuple<T, U>(arr: Fuzz<T, U>[]): Fuzz<T[], U[]>;
    /**
     * Creates a fuzzer from an object where the values of the object are
     * either fuzzers or plain values. If they are plain values, they are
     * considered constant values and will not shrink.
     * @param obj Object of fuzzers
     */
    object<T>(obj: {
        [K in keyof T]: T[K] | Fuzz<any, T[K]>;
    }): Fuzz<any, T>;
}
declare const _default: Api;
export default _default;
